<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Price Per Unit by Amasty
 */

namespace Amasty\PricePerUnit\Block\Adminhtml\Form;

use Amasty\PricePerUnit\Api\Data\UnitEntityInterface;
use Amasty\PricePerUnit\Controller\Adminhtml\Unit\Delete as DeleteController;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\AuthorizationInterface;
use Magento\Framework\Escaper;
use Magento\Framework\UrlInterface;
use Magento\Framework\View\Element\UiComponent\Control\ButtonProviderInterface;

/**
 * Delete entity button.
 */
class Delete implements ButtonProviderInterface
{
    /**
     * @var UrlInterface
     */
    private UrlInterface $urlBuilder;

    /**
     * @var Escaper
     */
    private Escaper $escaper;

    /**
     * @var RequestInterface
     */
    private RequestInterface $request;

    /**
     * @var AuthorizationInterface
     */
    private AuthorizationInterface $authorization;

    public function __construct(
        UrlInterface $urlBuilder,
        Escaper $escaper,
        RequestInterface $request,
        AuthorizationInterface $authorization
    ) {
        $this->urlBuilder = $urlBuilder;
        $this->escaper = $escaper;
        $this->request = $request;
        $this->authorization = $authorization;
    }

    /**
     * @return array
     */
    public function getButtonData(): array
    {
        $entityId = $this->getId();
        if (!$entityId || !$this->authorization->isAllowed(DeleteController::ADMIN_RESOURCE)) {
            return [];
        }

        return [
            'label' => __('Delete'),
            'class' => 'delete',
            'on_click' => sprintf(
                'deleteConfirm("%s", "%s")',
                $this->escaper->escapeHtml(__('Are you sure you want to delete?')->render()),
                $this->urlBuilder->getUrl('*/*/delete', ['id' => $entityId])
            ),
            'sort_order' => 20,
        ];
    }

    private function getId(): int
    {
        return (int)$this->request->getParam(UnitEntityInterface::ID);
    }
}
