<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Price Per Unit by Amasty
 */

namespace Amasty\PricePerUnit\Block;

use Amasty\PricePerUnit\Model\Product\AttributeData;
use Amasty\PricePerUnit\Model\TemplateProcessor;
use Amasty\PricePerUnit\Model\UnitEntity\Get;
use Magento\Framework\Pricing\PriceCurrencyInterface;
use Magento\Framework\Pricing\Render\AbstractAdjustment;
use Magento\Framework\Pricing\SaleableInterface;
use Magento\Framework\View\Element\Template;

class UnitPrice extends AbstractAdjustment
{
    public const CODE = 'am_unit';

    public const SALEABLE_ITEM_KEY = 'saleable_item';

    public const PRODUCT_PRICE_KEY = 'product_price';

    /**
     * @var string
     */
    protected $_template = 'Amasty_PricePerUnit::unit_price.phtml';

    /**
     * @var TemplateProcessor
     */
    private TemplateProcessor $templateProcessor;

    /**
     * @var Get
     */
    private Get $getUnit;

    public function __construct(
        Template\Context $context,
        PriceCurrencyInterface $priceCurrency,
        TemplateProcessor $templateProcessor,
        Get $getUnit,
        array $data = []
    ) {
        parent::__construct($context, $priceCurrency, $data);
        $this->templateProcessor = $templateProcessor;
        $this->getUnit = $getUnit;
    }

    protected function apply()
    {
        $item = $this->getSaleableItem();
        if (!(int)$item->getData(AttributeData::MEASURING_UNIT_ATTRIBUTE)
            || !(float)$item->getData(AttributeData::AMOUNT_ATTRIBUTE)
            || !$this->getProductPrice()
        ) {
            return '';
        }
        return $this->toHtml();
    }

    public function getTemplateHtml(): string
    {
        $item = $this->getSaleableItem();
        $unitId = (int)$item->getData(AttributeData::MEASURING_UNIT_ATTRIBUTE);
        $amount = (float)$item->getData(AttributeData::AMOUNT_ATTRIBUTE);
        $unitEntity = $this->getUnit->execute($unitId);

        return $this->templateProcessor->processUnitTemplate($unitEntity, $this->getProductPrice(), $amount);
    }

    public function getSaleableItem(): SaleableInterface
    {
        if ($this->getData(self::SALEABLE_ITEM_KEY)) {
            return $this->getData(self::SALEABLE_ITEM_KEY);
        }
        return parent::getSaleableItem();
    }

    public function getProductPrice(): float
    {
        if ($this->getData(self::PRODUCT_PRICE_KEY)) {
            return (float)$this->getData(self::PRODUCT_PRICE_KEY);
        }
        return (float)$this->getPrice()->getAmount()->getValue();
    }

    public function getAdjustmentCode()
    {
        return self::CODE;
    }
}
