<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Price Per Unit by Amasty
 */

namespace Amasty\PricePerUnit\Controller\Adminhtml\Unit;

use Amasty\PricePerUnit\Model\UnitEntity\Delete as UnitDelete;
use Exception;
use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Backend\Model\View\Result\Redirect;
use Magento\Framework\App\Action\HttpGetActionInterface;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Exception\LocalizedException;
use Psr\Log\LoggerInterface;

class Delete extends Action implements HttpGetActionInterface
{
    public const ADMIN_RESOURCE = 'Amasty_PricePerUnit::delete';

    /**
     * @var LoggerInterface
     */
    private LoggerInterface $logger;

    /**
     * @var UnitDelete
     */
    private UnitDelete $unitDelete;

    public function __construct(Context $context, LoggerInterface $logger, UnitDelete $unitDelete)
    {
        parent::__construct($context);
        $this->logger = $logger;
        $this->unitDelete = $unitDelete;
    }

    public function execute()
    {
        /** @var Redirect $redirect */
        $redirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);

        $entityId = (int) $this->getRequest()->getParam('id');
        if ($entityId) {
            try {
                $this->unitDelete->deleteById($entityId);
                $this->messageManager->addSuccessMessage(__('Unit was deleted successfully.'));
                return $redirect->setPath('*/*');
            } catch (LocalizedException $e) {
                $this->messageManager->addErrorMessage($e->getMessage());
            } catch (Exception $e) {
                $this->messageManager->addErrorMessage(__('Something went wrong. Please review the error log.'));
                $this->logger->error($e);
            }
            return $redirect->setRefererUrl();
        }

        return $redirect->setPath('*/*');
    }
}
