<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Price Per Unit by Amasty
 */

namespace Amasty\PricePerUnit\Controller\Adminhtml\Unit;

use Amasty\PricePerUnit\Model\ResourceModel\UnitEntity\Collection;
use Amasty\PricePerUnit\Model\ResourceModel\UnitEntity\CollectionFactory;
use Amasty\PricePerUnit\Model\UnitEntity\Delete as UnitDelete;
use Exception;
use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Exception\LocalizedException;
use Magento\Ui\Component\MassAction\Filter;
use Psr\Log\LoggerInterface;

class MassDelete extends Action implements HttpPostActionInterface
{
    public const ADMIN_RESOURCE = Delete::ADMIN_RESOURCE;
    /**
     * @var LoggerInterface
     */
    private LoggerInterface $logger;

    /**
     * @var UnitDelete
     */
    private UnitDelete $unitDelete;

    /**
     * @var Filter
     */
    private Filter $filter;

    /**
     * @var CollectionFactory
     */
    private CollectionFactory $collectionFactory;

    public function __construct(
        Context $context,
        LoggerInterface $logger,
        UnitDelete $unitDelete,
        Filter $filter,
        CollectionFactory $collectionFactory
    ) {
        parent::__construct($context);
        $this->logger = $logger;
        $this->unitDelete = $unitDelete;
        $this->filter = $filter;
        $this->collectionFactory = $collectionFactory;
    }

    public function execute()
    {
        $collection = $this->retrieveCollection();

        if ($collection && $collection->getSize()) {
            $affectedItems = 0;

            /** @var \Amasty\PricePerUnit\Model\UnitEntity $model */
            foreach ($collection->getItems() as $model) {
                try {
                    $this->unitDelete->execute($model);
                    $affectedItems++;
                } catch (LocalizedException $e) {
                    $this->messageManager->addErrorMessage($e->getMessage());
                } catch (Exception $e) {
                    $this->messageManager->addErrorMessage(
                        __('We can\'t change item right now. Please review the log and try again.')
                    );
                    $this->logger->critical($e);
                }
            }

            $this->messageManager->addSuccessMessage(__('A total of %1 record(s) have been deleted.', $affectedItems));
        }

        /** @var \Magento\Framework\Controller\Result\Redirect $redirect */
        $redirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);

        return $redirect->setRefererUrl();
    }

    private function retrieveCollection(): ?Collection
    {
        try {
            /** @var Collection $collection */
            $collection = $this->filter->getCollection($this->collectionFactory->create());
        } catch (LocalizedException $e) {
            $this->messageManager->addErrorMessage($e->getMessage());
            $collection = null;
        }

        return $collection;
    }
}
