<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Price Per Unit by Amasty
 */

namespace Amasty\PricePerUnit\Controller\Adminhtml\Unit;

use Amasty\PricePerUnit\Model\UnitEntity\Get as GetCommand;
use Amasty\PricePerUnit\Model\UnitEntity\Save as SaveCommand;
use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Backend\Model\View\Result\Redirect;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\App\Request\DataPersistorInterface;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\Exception\LocalizedException;
use Psr\Log\LoggerInterface;

class Save extends Action implements HttpPostActionInterface
{
    public const PERSISTENT_NAME = 'amasty_unit';

    /**
     * @var GetCommand
     */
    private GetCommand $unitGetter;

    /**
     * @var SaveCommand
     */
    private SaveCommand $unitSaver;

    /**
     * @var LoggerInterface
     */
    private LoggerInterface $logger;

    /**
     * @var DataPersistorInterface
     */
    private DataPersistorInterface $dataPersistor;

    public function __construct(
        Context $context,
        GetCommand $unitGetter,
        SaveCommand $unitSaver,
        LoggerInterface $logger,
        DataPersistorInterface $dataPersistor
    ) {
        parent::__construct($context);
        $this->unitGetter = $unitGetter;
        $this->unitSaver = $unitSaver;
        $this->logger = $logger;
        $this->dataPersistor = $dataPersistor;
    }

    /**
     * @return ResultInterface|ResponseInterface
     */
    public function execute()
    {
        $data = $this->getRequest()->getPostValue('unit');
        $entityId = (int)($data['id'] ?? 0);
        /** @var Redirect $redirect */
        $redirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
        $redirect->setPath('*/*/index');

        $this->dataPersistor->set(self::PERSISTENT_NAME, $data);

        try {
            unset($data['id']);
            $unitEntity = $this->unitGetter->execute($entityId);
            $unitEntity->addData($data);

            $this->unitSaver->execute($unitEntity);
        } catch (LocalizedException $e) {
            $this->messageManager->addErrorMessage($e->getMessage());

            return $redirect;
        } catch (\Exception $e) {
            $this->messageManager->addErrorMessage(__('Something went wrong. Please review the error log.'));
            $this->logger->error($e->getMessage());

            return $redirect;
        }

        $this->messageManager->addSuccessMessage(__('Unit was saved successfully.'));

        if ($this->getRequest()->getParam('back')) {
            $redirect->setPath('*/*/edit', ['id' => $unitEntity->getId()]);
        }
        $this->dataPersistor->clear(self::PERSISTENT_NAME);

        return $redirect;
    }

    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed(Edit::ADMIN_RESOURCE)
            || $this->_authorization->isAllowed(NewAction::ADMIN_RESOURCE);
    }
}
