<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Price Per Unit by Amasty
 */

namespace Amasty\PricePerUnit\Model\Product;

use Magento\Catalog\Model\Product;
use Magento\Eav\Model\Config;
use Magento\Framework\App\ResourceConnection;

class ClearAttributeData
{
    /**
     * @var ResourceConnection
     */
    private ResourceConnection $resourceConnection;

    /**
     * @var Config
     */
    private Config $eavConfig;

    public function __construct(
        ResourceConnection $resourceConnection,
        Config $eavConfig
    ) {
        $this->resourceConnection = $resourceConnection;
        $this->eavConfig = $eavConfig;
    }

    public function execute(int $unitId): void
    {
        $measuringUnitAttribute = $this->eavConfig->getAttribute(
            Product::ENTITY,
            AttributeData::MEASURING_UNIT_ATTRIBUTE
        );
        if ($measuringUnitAttribute->getAttributeId()) {
            $connection = $this->resourceConnection->getConnection();
            $where = $connection->quoteInto('attribute_id = ?', $measuringUnitAttribute->getAttributeId());
            $where.= $connection->quoteInto(' AND value = ?', $unitId);
            $connection->delete($measuringUnitAttribute->getBackendTable(), $where);
        }
    }
}
