<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Price Per Unit by Amasty
 */

namespace Amasty\PricePerUnit\Model\Product;

use Amasty\PricePerUnit\Model\UnitEntity\GetList;
use Magento\Catalog\Model\ResourceModel\Product\Collection;
use Magento\Framework\Api\SearchCriteriaBuilder;

class PreloadUnitEntities
{
    /**
     * @var SearchCriteriaBuilder
     */
    private SearchCriteriaBuilder $searchCriteriaBuilder;

    /**
     * @var GetList
     */
    private GetList $getList;

    public function __construct(
        SearchCriteriaBuilder $searchCriteriaBuilder,
        GetList $getList
    ) {
        $this->searchCriteriaBuilder =  $searchCriteriaBuilder;
        $this->getList = $getList;
    }

    public function execute(Collection $productCollection): void
    {
        $unitEntityIds = $productCollection->getAllAttributeValues(AttributeData::MEASURING_UNIT_ATTRIBUTE);
        $unitEntityIds = array_unique(array_map('current', $unitEntityIds));

        $this->searchCriteriaBuilder->addFilter('id', $unitEntityIds, 'in');
        $searchCriteria = $this->searchCriteriaBuilder->create();
        $this->getList->getList($searchCriteria);
    }
}
