<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Price Per Unit by Amasty
 */

namespace Amasty\PricePerUnit\Model\Source;

use Amasty\PricePerUnit\Api\Data\UnitEntityInterface;
use Amasty\PricePerUnit\Model\ResourceModel\UnitEntity;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Data\OptionSourceInterface;

class ColumnOptions implements OptionSourceInterface
{
    /**
     * @var ResourceConnection
     */
    private ResourceConnection $resourceConnection;

    /**
     * @var string
     */
    private string $columnValue;

    /**
     * @var string
     */
    private string $columnLabel;

    public function __construct(
        ResourceConnection $resourceConnection,
        string $columnValue = UnitEntityInterface::ID,
        string $columnLabel = UnitEntityInterface::NAME
    ) {
        $this->resourceConnection = $resourceConnection;
        $this->columnValue = $columnValue;
        $this->columnLabel = $columnLabel;
    }

    public function toOptionArray(): array
    {
        $connection = $this->resourceConnection->getConnection();
        $select = $connection->select()->from(
            $this->resourceConnection->getTableName(UnitEntity::TABLE_NAME),
            ['value' => $this->columnValue, 'label' => $this->columnLabel]
        );
        $select->distinct();

        return array_values($connection->fetchAssoc($select) ?? []);
    }
}
