<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Price Per Unit by Amasty
 */

namespace Amasty\PricePerUnit\Model;

use Amasty\PricePerUnit\Api\Data\UnitEntityInterface;
use Magento\Framework\Escaper;
use Magento\Framework\Pricing\PriceCurrencyInterface;

class TemplateProcessor
{
    /**
     * @var Escaper
     */
    private Escaper $escaper;

    /**
     * @var PriceCurrencyInterface
     */
    private PriceCurrencyInterface $currency;

    /**
     * @var UnitCalculator
     */
    private UnitCalculator $unitCalculator;

    public function __construct(Escaper $escaper, PriceCurrencyInterface $currency, UnitCalculator $unitCalculator)
    {
        $this->escaper = $escaper;
        $this->currency = $currency;
        $this->unitCalculator = $unitCalculator;
    }

    public function processUnitTemplate(UnitEntityInterface $unit, float $productPrice, float $productAmount): string
    {
        $template = $unit->getTextTemplate();
        $unitPrice = $this->unitCalculator->calculate($unit, $productPrice, $productAmount);

        $map = [
            '{{unit_price}}' => $this->currency->format($unitPrice),
            '{{reference_amount}}' => $unit->getReferenceAmount(),
            '{{reference_unit}}' => $unit->getReferenceUnit(),
            '{{measure_unit}}' => $unit->getMeasureUnit(),
            '{{product_amount}}' => $productAmount,
            '{{product_price}}' => $this->currency->format($productPrice, false),
        ];
        foreach ($map as $variableTemplate => $value) {
            $template = str_replace(
                $variableTemplate,
                (string)$value,
                $template
            );
        }

        return $this->escaper->escapeHtml($template, ['b', 's', 'i', 'u', 'strong', 'span']);
    }
}
