<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Price Per Unit by Amasty
 */

namespace Amasty\PricePerUnit\Model\UnitEntity;

use Amasty\PricePerUnit\Api\Data\UnitEntityInterface;
use Amasty\PricePerUnit\Model\ResourceModel\UnitEntity as ResourceUnitEntity;
use Amasty\PricePerUnit\Model\UnitEntity;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Exception\NoSuchEntityException;

class Delete
{
    /**
     * @var ResourceUnitEntity
     */
    private ResourceUnitEntity $resourceUnitEntity;

    /**
     * @var Get
     */
    private Get $get;

    public function __construct(
        ResourceUnitEntity $resourceUnitEntity,
        Get $get
    ) {
        $this->resourceUnitEntity = $resourceUnitEntity;
        $this->get = $get;
    }

    /**
     * @throws CouldNotDeleteException
     */
    public function execute(UnitEntityInterface $unitEntity): void
    {
        try {
            $this->resourceUnitEntity->delete($unitEntity);
        } catch (\Exception $exception) {
            throw new CouldNotDeleteException(
                __(
                    'Could not delete the Unit Entity: %1',
                    $exception->getMessage()
                )
            );
        }
    }

    public function deleteById(int $entityId): void
    {
        try {
            $unitEntity = $this->get->execute($entityId);
        } catch (NoSuchEntityException $exception) {
            return;
        }
        $this->execute($unitEntity);
    }
}
