<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Price Per Unit by Amasty
 */

namespace Amasty\PricePerUnit\Model\UnitEntity;

use Amasty\PricePerUnit\Model\ResourceModel\UnitEntity as ResourceUnitEntity;
use Amasty\PricePerUnit\Model\UnitEntity;
use Amasty\PricePerUnit\Model\UnitEntityFactory;
use Magento\Framework\Exception\NoSuchEntityException;

class Get
{
    /**
     * @var UnitEntityFactory
     */
    private UnitEntityFactory $unitEntityFactory;

    /**
     * @var ResourceUnitEntity
     */
    private ResourceUnitEntity $resourceUnitEntity;

    /**
     * @var Registry
     */
    private Registry $registry;

    public function __construct(
        UnitEntityFactory $unitEntityFactory,
        ResourceUnitEntity $resourceUnitEntity,
        Registry $registry
    ) {
        $this->unitEntityFactory = $unitEntityFactory;
        $this->resourceUnitEntity = $resourceUnitEntity;
        $this->registry = $registry;
    }

    /**
     * @throws NoSuchEntityException
     */
    public function execute(int $entityId): UnitEntity
    {
        if ($entityId === 0) {
            return $this->unitEntityFactory->create();
        }
        if (!isset($this->registry->isset[$entityId])) {
            $unitEntity = $this->unitEntityFactory->create();
            $this->resourceUnitEntity->load($unitEntity, $entityId);
            if ($unitEntity->getId() === null) {
                throw new NoSuchEntityException(__('Unit entity with id "%1" does not exist.', $entityId));
            }
            $this->registry->set($entityId, $unitEntity);
        }

        return $this->registry->get($entityId);
    }
}
