<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Price Per Unit by Amasty
 */

namespace Amasty\PricePerUnit\Model\UnitEntity;

use Amasty\PricePerUnit\Api\Data\UnitEntitySearchResultsInterface;
use Amasty\PricePerUnit\Api\Data\UnitEntitySearchResultsInterfaceFactory;
use Amasty\PricePerUnit\Model\ResourceModel\UnitEntity\CollectionFactory;
use Magento\Framework\Api\SearchCriteria;
use Magento\Framework\Api\SearchCriteria\CollectionProcessorInterface;
use Magento\Framework\Api\SearchResultsInterfaceFactory;

class GetList
{
    /**
     * @var CollectionFactory
     */
    private CollectionFactory $collectionFactory;

    /**
     * @var CollectionProcessorInterface
     */
    private CollectionProcessorInterface $collectionProcessor;

    /**
     * @var Registry
     */
    private Registry $registry;

    /**
     * @var UnitEntitySearchResultsInterfaceFactory
     */
    private UnitEntitySearchResultsInterfaceFactory $searchResultsFactory;

    public function __construct(
        CollectionProcessorInterface $collectionProcessor,
        CollectionFactory $collectionFactory,
        UnitEntitySearchResultsInterfaceFactory $searchResultsFactory,
        Registry $registry
    ) {
        $this->collectionProcessor = $collectionProcessor;
        $this->collectionFactory = $collectionFactory;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->registry = $registry;
    }

    public function getList(SearchCriteria $searchCriteria): UnitEntitySearchResultsInterface
    {
        $collection = $this->collectionFactory->create();
        $this->collectionProcessor->process($searchCriteria, $collection);

        foreach ($collection->getItems() as $unitEntity) {
            $this->registry->set($unitEntity->getId(), $unitEntity);
        }

        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($searchCriteria);
        $searchResults->setItems($collection->getItems());
        $searchResults->setTotalCount($collection->getSize());

        return $searchResults;
    }
}
