<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Price Per Unit by Amasty
 */

namespace Amasty\PricePerUnit\Plugin\Checkout\Block\Cart\Item\Renderer;

use Amasty\PricePerUnit\Block\UnitPrice;
use Amasty\PricePerUnit\Model\ConfigProvider;
use Amasty\PricePerUnit\Model\Product\AttributeData;
use Magento\Catalog\Model\Product\Type;
use Magento\Checkout\Block\Cart\Item\Renderer;
use Magento\ConfigurableProduct\Model\Product\Type\Configurable;
use Magento\Quote\Model\Quote\Item\AbstractItem;
use Magento\Tax\Model\Config;

class AddUnitPrice
{
    /**
     * @var ConfigProvider
     */
    private ConfigProvider $configProvider;

    /**
     * @var Config
     */
    private Config $taxConfig;

    public function __construct(ConfigProvider $configProvider, Config $taxConfig)
    {
        $this->configProvider = $configProvider;
        $this->taxConfig = $taxConfig;
    }

    /**
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function afterGetUnitPriceHtml(Renderer $subject, string $result, AbstractItem $item): string
    {
        $product = $item->getProduct();
        if (in_array($product->getTypeId(), [Type::TYPE_SIMPLE, Configurable::TYPE_CODE], true)
            && $product->getData(AttributeData::MEASURING_UNIT_ATTRIBUTE)
            && $this->configProvider->isEnabled()
        ) {
            $price = $this->taxConfig->displayCartPricesExclTax()
                ? $item->getCalculationPrice()
                : $item->getPriceInclTax();
            $unitBlock = $subject->getLayout()->createBlock(UnitPrice::class, '', [
                'data' => [
                    UnitPrice::SALEABLE_ITEM_KEY => $product,
                    UnitPrice::PRODUCT_PRICE_KEY => $price,
                ]
            ]);
            $result .= $unitBlock->toHtml();
        }

        return $result;
    }
}
