<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Price Per Unit by Amasty
 */

namespace Amasty\PricePerUnit\Plugin\Framework\Pricing\RenderPool;

use Amasty\PricePerUnit\Block\UnitPrice;
use Amasty\PricePerUnit\Model\ConfigProvider;
use Amasty\PricePerUnit\Model\Product\AttributeData;
use Magento\Catalog\Model\Product\Type;
use Magento\ConfigurableProduct\Model\Product\Type\Configurable;
use Magento\Framework\Pricing\Price\PriceInterface;
use Magento\Framework\Pricing\Render\RendererPool;
use Magento\Framework\Pricing\SaleableInterface;

class AddUnitRender
{
    /**
     * @var ConfigProvider
     */
    private ConfigProvider $configProvider;

    public function __construct(ConfigProvider $configProvider)
    {
        $this->configProvider = $configProvider;
    }

    /**
     * @param RendererPool $subject
     * @param array $result
     * @param SaleableInterface|null $saleableItem
     * @param PriceInterface|null $price
     * @return array
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function afterGetAdjustmentRenders(
        RendererPool $subject,
        array $result,
        SaleableInterface $saleableItem = null,
        PriceInterface $price = null
    ): array {
        if ($saleableItem !== null
            && in_array($saleableItem->getTypeId(), [Type::TYPE_SIMPLE, Configurable::TYPE_CODE], true)
            && $saleableItem->getData(AttributeData::MEASURING_UNIT_ATTRIBUTE)
            && $this->configProvider->isEnabled()
        ) {
            $result[UnitPrice::CODE] = $subject->getLayout()->createBlock(UnitPrice::class);
        }

        return $result;
    }
}
