<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Price Per Unit by Amasty
 */

namespace Amasty\PricePerUnit\Setup\Patch\Data;

use Amasty\PricePerUnit\Model\Product\AttributeData;
use Amasty\PricePerUnit\Model\Source\Unit;
use Magento\Catalog\Model\Product;
use Magento\Eav\Setup\EavSetup;
use Magento\Eav\Setup\EavSetupFactory;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\Patch\DataPatchInterface;
use Magento\Framework\Setup\Patch\PatchRevertableInterface;

class AddProductAttributes implements DataPatchInterface, PatchRevertableInterface
{
    /**
     * @var EavSetup
     */
    private EavSetup $eavSetup;

    public function __construct(
        EavSetupFactory $eavSetupFactory,
        ModuleDataSetupInterface $setup
    ) {
        $this->eavSetup = $eavSetupFactory->create(['setup' => $setup]);
    }

    public static function getDependencies(): array
    {
        return [];
    }

    public function getAliases(): array
    {
        return [];
    }

    public function apply(): self
    {
        $measuringUnitAttributeData = [
            'type' => 'int',
            'label' => 'Product Measuring Unit',
            'input' => 'select',
            'source' => Unit::class,
        ];
        $this->addAttribute(AttributeData::MEASURING_UNIT_ATTRIBUTE, $measuringUnitAttributeData);

        $amountAttributeData = [
            'type' => 'decimal',
            'label' => 'Product Amount',
            'input' => 'text',
        ];
        $this->addAttribute(AttributeData::AMOUNT_ATTRIBUTE, $amountAttributeData);

        return $this;
    }

    public function revert(): void
    {
        $this->eavSetup->removeAttribute(Product::ENTITY, AttributeData::MEASURING_UNIT_ATTRIBUTE);
        $this->eavSetup->removeAttribute(Product::ENTITY, AttributeData::AMOUNT_ATTRIBUTE);
    }

    private function addAttribute(string $attributeName, array $attributeData): void
    {
        if (!$this->eavSetup->getAttribute(Product::ENTITY, $attributeName)) {
            $attributeData = array_merge(
                $attributeData,
                [
                    'backend' => '',
                    'frontend' => '',
                    'visible' => true,
                    'required' => false,
                    'user_defined' => false,
                    'used_in_product_listing' => true,
                    'searchable' => false,
                    'filterable' => false,
                    'comparable' => false,
                    'visible_on_front' => false,
                    'unique' => false,
                    'apply_to' => 'simple,configurable',
                    'group' => 'Unit Price'
                ]
            );
            $this->eavSetup->addAttribute(
                Product::ENTITY,
                $attributeName,
                $attributeData
            );
        }
    }
}
