<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Price Per Unit by Amasty
 */

namespace Amasty\PricePerUnit\Setup\Patch\Data;

use Amasty\PricePerUnit\Api\Data\UnitEntityInterface;
use Amasty\PricePerUnit\Model\ResourceModel\UnitEntity;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\Patch\DataPatchInterface;

class SampleUnitsData implements DataPatchInterface
{
    private const UNITS_DATA = [
        [
            UnitEntityInterface::NAME => 'Grams/Kilograms',
            UnitEntityInterface::MEASURE_UNIT => 'Gr',
            UnitEntityInterface::REFERENCE_UNIT => 'Kg',
            UnitEntityInterface::REFERENCE_AMOUNT => 1,
            UnitEntityInterface::CONVERSION_RATE => 1000,
            UnitEntityInterface::TEXT_TEMPLATE => '{{unit_price}} / {{reference_amount}} {{reference_unit}}',
        ],
        [
            UnitEntityInterface::NAME => 'Milliliters/Liters',
            UnitEntityInterface::MEASURE_UNIT => 'Ml',
            UnitEntityInterface::REFERENCE_UNIT => 'L',
            UnitEntityInterface::REFERENCE_AMOUNT => 1,
            UnitEntityInterface::CONVERSION_RATE => 1000,
            UnitEntityInterface::TEXT_TEMPLATE => '{{unit_price}} / {{reference_amount}} {{reference_unit}}',
        ],
        [
            UnitEntityInterface::NAME => 'Centimeters/Meters',
            UnitEntityInterface::MEASURE_UNIT => 'Cm',
            UnitEntityInterface::REFERENCE_UNIT => 'M',
            UnitEntityInterface::REFERENCE_AMOUNT => 1,
            UnitEntityInterface::CONVERSION_RATE => 100,
            UnitEntityInterface::TEXT_TEMPLATE => '{{unit_price}} / {{reference_amount}} {{reference_unit}}',
        ]
    ];

    /**
     * @var ModuleDataSetupInterface
     */
    private ModuleDataSetupInterface $moduleDataSetup;

    public function __construct(
        ModuleDataSetupInterface $moduleDataSetup
    ) {
        $this->moduleDataSetup = $moduleDataSetup;
    }

    public static function getDependencies(): array
    {
        return [];
    }

    public function getAliases(): array
    {
        return [];
    }

    public function apply(): self
    {
        $connection = $this->moduleDataSetup->getConnection();

        $connection->insertMultiple(
            $this->moduleDataSetup->getTable(UnitEntity::TABLE_NAME),
            self::UNITS_DATA
        );

        return $this;
    }
}
