<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Price Per Unit by Amasty
 */

namespace Amasty\PricePerUnit\Test\Integration;

use Amasty\PricePerUnit\Api\Data\UnitEntityInterface;
use Amasty\PricePerUnit\Model\Product\AttributeData;
use Amasty\PricePerUnit\Model\ResourceModel\UnitEntity;
use Amasty\PricePerUnit\Model\UnitEntityFactory;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\TestFramework\TestCase\AbstractController;

class RenderBlockTest extends AbstractController
{
    private const PRODUCT_SKU = 'simple';
    private const UNIT_CLASS_STRING = 'class="am-price-per-unit"';
    private const PRODUCT_URL = 'catalog/product/view/id/%s';

    /**
     * @var ProductInterface|null
     */
    private ?ProductInterface $product;

    /**
     * @var ProductRepositoryInterface|null
     */
    private ?ProductRepositoryInterface $productRepository;

    protected function setUp(): void
    {
        parent::setUp();

        $this->productRepository = $this->_objectManager->create(ProductRepositoryInterface::class);
        $this->product = $this->productRepository->get(self::PRODUCT_SKU);
    }

    /**
     * @magentoConfigFixture current_store amasty_price_per_unit/general/enable 1
     * @magentoDataFixture Magento/Catalog/_files/products.php
     * @magentoDataFixture Amasty_PricePerUnit::Test/Integration/_files/units.php
     */
    public function testRenderProductWithUnitDataAndEnableUnit(): void
    {
        $this->assignUnitToProduct();
        $this->dispatch(sprintf(self::PRODUCT_URL, $this->product->getEntityId()));
        $responseBody = $this->getResponse()->getBody();

        $this->assertStringContainsString(self::UNIT_CLASS_STRING, $responseBody);
    }

    /**
     * @magentoConfigFixture current_store amasty_price_per_unit/general/enable 0
     * @magentoDataFixture Magento/Catalog/_files/products.php
     * @magentoDataFixture Amasty_PricePerUnit::Test/Integration/_files/units.php
     */
    public function testRenderProductWithUnitDataAndDisableUnit(): void
    {
        $this->assignUnitToProduct();
        $this->dispatch(sprintf(self::PRODUCT_URL, $this->product->getEntityId()));
        $responseBody = $this->getResponse()->getBody();

        $this->assertStringNotContainsString(self::UNIT_CLASS_STRING, $responseBody);
    }

    /**
     * @magentoConfigFixture current_store amasty_price_per_unit/general/enable 1
     * @magentoDataFixture Magento/Catalog/_files/products.php
     */
    public function testRenderProductWithoutUnitDataAndEnableUnit(): void
    {
        $this->dispatch(sprintf(self::PRODUCT_URL, $this->product->getEntityId()));
        $responseBody = $this->getResponse()->getBody();

        $this->assertStringNotContainsString(self::UNIT_CLASS_STRING, $responseBody);
    }

    private function assignUnitToProduct(): void
    {
        $unit = $this->_objectManager->create(UnitEntityFactory::class)->create();
        $unitRepository = $this->_objectManager->create(UnitEntity::class);
        $unitRepository->load($unit, 'unit', UnitEntityInterface::NAME);

        $this->product->setData(AttributeData::MEASURING_UNIT_ATTRIBUTE, $unit->getId());
        $this->product->setData(AttributeData::AMOUNT_ATTRIBUTE, 1000);
        $this->productRepository->save($this->product);
    }
}
