<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Price Per Unit by Amasty
 */

namespace Amasty\PricePerUnit\Test\Unit\Model;

use Amasty\PricePerUnit\Model\UnitCalculator;
use Amasty\PricePerUnit\Model\UnitEntity;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;

/**
 * @see UnitCalculator
 */
class UnitCalculatorTest extends TestCase
{
    /**
     * @var UnitEntity|MockObject
     */
    private $unitEntityMock;

    /**
     * @var UnitCalculator
     */
    private UnitCalculator $subject;

    protected function setUp(): void
    {
        $this->unitEntityMock = $this->createMock(UnitEntity::class);

        $this->subject = new UnitCalculator();
    }

    /**
     * @dataProvider calculateDataProvider
     *
     * @covers UnitCalculator::calculate
     */
    public function testCalculate(
        float $conversionRate,
        float $referenceAmount,
        float $productPrice,
        float $productAmount,
        float $result
    ): void {
        $this->unitEntityMock->method('getConversionRate')->willReturn($conversionRate);
        $this->unitEntityMock->method('getReferenceAmount')->willReturn($referenceAmount);

        $this->assertEquals(
            $result,
            round($this->subject->calculate($this->unitEntityMock, $productPrice, $productAmount), 2)
        );
    }

    public function calculateDataProvider(): array
    {
        return [
            [0.1, 1.00, 0.00, 10.00, 0.00],
            [0.1, 1.00, 10.00, 0.00, 0.00],
            [0.1, 1.00, 10.00, 10.00, 0.1],
            [0.0001, 1.00, 10.00, 100.00, 0.00],
            [5.00, 1.00, 3.00, 10.21, 1.47],
            [1000.0, 5.00, 0.0002, 10.0, 0.1]
        ];
    }
}
