<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Price Per Unit by Amasty
 */

namespace Amasty\PricePerUnit\Ui;

use Amasty\PricePerUnit\Api\Data\UnitEntityInterface;
use Amasty\PricePerUnit\Controller\Adminhtml\Unit\Save;
use Amasty\PricePerUnit\Model\UnitEntity;
use Magento\Framework\Api\FilterBuilder;
use Magento\Framework\Api\Search\ReportingInterface;
use Magento\Framework\Api\Search\SearchCriteriaBuilder;
use Magento\Framework\App\Request\DataPersistorInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\View\Element\UiComponent\DataProvider\DataProvider as AbstractDataProvider;

/**
 * DataProvider component.
 */
class DataProvider extends AbstractDataProvider
{
    /**
     * @var DataPersistorInterface
     */
    private DataPersistorInterface $dataPersistor;

    /**
     * @var array|null
     */
    private ?array $loadedData = null;

    /**
     * @SuppressWarnings(PHPMD.ExcessiveParameterList)
     */
    public function __construct(
        $name,
        $primaryFieldName,
        $requestFieldName,
        ReportingInterface $reporting,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        RequestInterface $request,
        FilterBuilder $filterBuilder,
        DataPersistorInterface $dataPersistor,
        array $meta = [],
        array $data = []
    ) {
        parent::__construct(
            $name,
            $primaryFieldName,
            $requestFieldName,
            $reporting,
            $searchCriteriaBuilder,
            $request,
            $filterBuilder,
            $meta,
            $data
        );
        $this->dataPersistor = $dataPersistor;
    }

    /**
     * Get data.
     *
     * @return array
     */
    public function getData()
    {
        if ($this->loadedData === null) {
            $data = $this->dataPersistor->get(Save::PERSISTENT_NAME);
            if ($data) {
                $this->dataPersistor->clear(Save::PERSISTENT_NAME);
                $this->setItemData($data);
            } else {
                /** @var \Magento\Framework\View\Element\UiComponent\DataProvider\Document $unit */
                foreach ($this->getSearchResult()->getItems() as $unit) {
                    $data = $unit->toArray();
                    $this->setItemData($data);
                }
                if ($this->loadedData === null) {
                    $this->loadedData[null]['unit'] = [];
                }
            }
        }

        return $this->loadedData;
    }

    private function setItemData(array $data): void
    {
        $entityId = $data[UnitEntityInterface::ID] ?? null;
        $data[UnitEntityInterface::REFERENCE_AMOUNT] = (float)$data[UnitEntityInterface::REFERENCE_AMOUNT];
        $data[UnitEntityInterface::CONVERSION_RATE] = (float)$data[UnitEntityInterface::CONVERSION_RATE];
        $this->loadedData[$entityId]['unit'] = $data;
    }
}
